--[[-----------------------------------------------------------------------------------------------

  HealerButtons
  http://www.wowinterface.com/portal.php?id=347
  Description: Adds up to three buttons next self/party frames for quick and convenient healing.

  Created by: Saiiyna of Lothar  
  ACE2 Conversion and Enhancements: CSilv

  $Id: HealerButtons.lua 3 2008-02-11 19:56:47Z chris $

---------------------------------------------------------------------------------------------- --]]

local L = AceLibrary("AceLocale-2.2"):new("HealerButtons")

HealerButtons = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")

HealerButtons.bandages = {
	Linen					= L["Linen Bandage"],
	HeavyLinen				= L["Heavy Linen Bandage"],
	Wool					= L["Wool Bandage"],
	HeavyWool				= L["Heavy Wool Bandage"],
	Silk					= L["Silk Bandage"],
	HeavySilk				= L["Heavy Silk Bandage"],
	Mageweave				= L["Mageweave Bandage"],
	HeavyMageweave			= L["Heavy Mageweave Bandage"],
	Runecloth				= L["Runecloth Bandage"],
	HeavyRunecloth			= L["Heavy Runecloth Bandage"],
	HighlandersRunecloth	= L["Highlander's Runecloth Bandage"],
	HighlandersMageweave	= L["Highlander's Mageweave Bandage"],
	HighlandersSilk			= L["Highlander's Silk Bandage"],
	DefilersRunecloth		= L["Defiler's Runecloth Bandage"],
	DefilersMageweave		= L["Defiler's Mageweave Bandage"],
	DefilersSilk			= L["Defiler's Silk Bandage"],
	ABRunecloth				= L["Arathi Basin Runecloth Bandage"],
	ABMageweave				= L["Arathi Basin Mageweave Bandage"],
	ABSilk					= L["Arathi Basin Silk Bandage"],
	WSGRunecloth			= L["Warsong Gulch Runecloth Bandage"],
	WSGMageweave 			= L["Warsong Gulch Mageweave Bandage"],
	WSGSilk					= L["Warsong Gulch Silk Bandage"],
	AVHeavyRunecloth		= L["Alterac Heavy Runecloth Bandage"],
	Netherweave				= L["Netherweave Bandage"],
	HeavyNetherweave		= L["Heavy Netherweave Bandage"]
}

HealerButtons.bandageInfo = {
	[HealerButtons.bandages.Linen]				= {Level=1, HealAmount=66, OnHand = 0, Bag = -1, Slot = 0},
	[HealerButtons.bandages.HeavyLinen]			= {Level=1, HealAmount=114, OnHand = 0, Bag = -1, Slot = 0},
	[HealerButtons.bandages.Wool]				= {Level=1, HealAmount=161, OnHand = 0, Bag = -1, Slot = 0},
	[HealerButtons.bandages.HeavyWool]			= {Level=1, HealAmount=301, OnHand = 0, Bag = -1, Slot = 0},
	[HealerButtons.bandages.Silk]				= {Level=1, HealAmount=400, OnHand = 0, Bag = -1, Slot = 0},
	[HealerButtons.bandages.HeavySilk]			= {Level=1, HealAmount=640, OnHand = 0, Bag = -1, Slot = 0},
	[HealerButtons.bandages.Mageweave]			= {Level=1, HealAmount=800, OnHand = 0, Bag = -1, Slot = 0},
	[HealerButtons.bandages.HeavyMageweave]		= {Level=1, HealAmount=1104, OnHand = 0, Bag = -1, Slot = 0},
	[HealerButtons.bandages.Runecloth]			= {Level=52,HealAmount=1360, OnHand = 0, Bag = -1, Slot = 0},
	[HealerButtons.bandages.HeavyRunecloth]		= {Level=58,HealAmount=2000, OnHand = 0, Bag = -1, Slot = 0},
	[HealerButtons.bandages.Netherweave]		= {Level=64,HealAmount=2800, OnHand = 0, Bag = -1, Slot = 0},
	[HealerButtons.bandages.HeavyNetherweave]	= {Level=70,HealAmount=3400, OnHand = 0, Bag = -1, Slot = 0}
}

HealerButtons.options = { 
	type = "group",
	args = {
		[L["reset"]] = {
			name = L["reset"],
			desc = L["Reset buttons to the default positions."],
			type = "execute",
			func = "ResetConfigSavedLocation"
		},
		[L["number"]] = {
			name = L["number"],
			desc = L["Set how many buttons should appear for each character frame."],
			type = "range", 
			min = 1, 
			max = 3, 
			step = 1,
			get = "GetButtonCount",
			set = "SetButtonCount"
		},
		[L["size"]] = {
			name = L["size"], 
			desc = L["Set the height and width of each button."],
			type = "range", 
			min = 10, 
			max = 20, 
			step = 1,
			get = "GetButtonSize",
			set = "SetButtonSize"
		},
		[L["movable"]] = {
			name = L["movable"],
			desc = L["Toggle whether buttons can be moved by dragging."],
			type = "toggle",
			get = "GetMovable",
			set = "SetMovable",
			map = {[false] = L["Off"], [true] = L["On"]}
		},
		[L["pets"]] = {
			name = L["pets"],
			desc = L["Toggle whether buttons are shown for party member pets."],
			type = "toggle",
			get = "GetShowPetButtons",
			set = "SetShowPetButtons",
			map = {[false] = L["Off"], [true] = L["On"]}
		},
		[L["tooltips"]] = {
			name = L["tooltips"],
			desc = L["Toggle whether buttons show tooltips on mouseover."],
			type = "toggle",
			get = "GetShowTooltips",
			set = "SetShowTooltips",
			map = {[false] = L["Off"], [true] = L["On"]}
		},
		[L["anchor"]] = {
			name = L["anchor"], 
			desc = L["Specify how tooltips are anchored to their parent button."],
			type = "text",
			get = "GetTooltipAnchor",
			set = "SetTooltipAnchor",
			validate = { ["TOPLEFT"] = L["TopLeft"], ["TOP"] = L["Top"], ["TOPRIGHT"] = L["TopRight"],
						 ["BOTTOMLEFT"] = L["BottomLeft"], ["BOTTOM"] = L["Bottom"], ["BOTTOMRIGHT"] = L["BottomRight"],
						 ["LEFT"] = L["Left"], ["RIGHT"] = L["Right"], ["CURSOR"] = L["Cursor"] }
		},
		[L["bind"]] = {
			name = L["bind"], 
			desc = L["Bind a spell or bandage to the mouse click of a button."],
			usage = L["nc <spell | bandage | none>\nOptions:\n     n - button number 1, 2, or 3\n     c - click l, r, or m\nExamples:\n     1l Lesser Heal  -  assigns Lesser Heal to the left click of button 1\n     1l Bandage  -  assigns best bandage to the left click of button 1\n     2r Linen Bandage  -  assigns Linen Bandage to the right click of button 2\n     2r none  -  clears the assignment of the right click of button 2"],
			type = "text",
			input = true,
			get = false,
			set = "SetBinding",
			validate = "VerifyBinding"
		}
    }
}

HealerButtons.PreferenceDefaults = {
	ShowTooltips		= true,
	TooltipAnchor		= "CURSOR",
	Movable				= true,
	ShowPetButtons		= true,
	ButtonCount			= 3,
	ButtonSize = { 18, 18 },
	ButtonLocs = {
		player		= {5, -15, 5, -35, 5, -55},
		party1		= {0, -8, 20, -8, 40, -8},
		party2		= {0, -8, 20, -8, 40, -8},
		party3		= {0, -8, 20, -8, 40, -8},
		party4		= {0, -8, 20, -8, 40, -8},
		target		= {5, -15, 5, -35, 5, -55},
		partypet1	= {-30, -38, -10, -38, 10, -38},
		partypet2	= {-30, -38, -10, -38, 10, -38},
		partypet3	= {-30, -38, -10, -38, 10, -38},
		partypet4	= {-30, -38, -10, -38, 10, -38}
	},
	SpellTable = {
		Priest = {
			Name = {
				L["Flash Heal"], L["Power Word: Shield"], "", L["Greater Heal"], L["Heal"], "", L["Renew"], "", ""
			},
			Type = {
				"SPELL","SPELL","SPELL","SPELL","SPELL","SPELL","SPELL","SPELL","SPELL"
			}
		},
		Shaman = {
			Name = {
				L["Healing Wave"], L["Lesser Healing Wave"], "", L["Chain Heal"], L["Ancestral Spirit"], "", L["Cure Disease"], L["Cure Poison"], ""
			},
			Type = {
				"SPELL","SPELL","SPELL","SPELL","SPELL","SPELL","SPELL","SPELL","SPELL"
			}
		},
		Druid = {
			Name = {
				L["Healing Touch"], L["Rejuvenation"], "", L["Rebirth"], L["Abolish Poison"], L["Cure Poison"], L["Regrowth"], L["Remove Curse"], ""
			},
			Type = {
				"SPELL","SPELL","SPELL","SPELL","SPELL","SPELL","SPELL","SPELL","SPELL"
			}
		},
		Paladin = {
			Name = {
				L["Flash of Light"], "", "", L["Holy Light"], "", "", L["Cleanse"], "", ""
			},
			Type = {
				"SPELL","SPELL","SPELL","SPELL","SPELL","SPELL","SPELL","SPELL","SPELL"
			}
		}
	}
}

HealerButtons:RegisterDB("Preferences")
HealerButtons:RegisterDefaults("profile", HealerButtons.PreferenceDefaults)

function HealerButtons:OnInitialize(name)
	self.Class = self:UnitClass("player")
	self.CanChangeButtons = true

	if (self:IsHealerClass()) then
		self:RegisterChatCommand(L["Slash-Commands"], HealerButtons.options)
	else
		self:RegisterChatCommand(L["Slash-Commands"])
	end
end

function HealerButtons:OnEnable(first)
	if self:IsHealerClass() then
		self:RegisterEvent("PARTY_MEMBERS_CHANGED")
		self:RegisterEvent("UNIT_PET")
		self:RegisterEvent("BAG_UPDATE")
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		self:RegisterEvent("PLAYER_REGEN_DISABLED")

		self:GetBandages()
		self:SetupButtons()
    end
end

function HealerButtons:OnDisable()
	if self:IsHealerClass() then
		self:HideButtons("player", "")
		for i = 1,4 do
			self:HideButtons("party", i)
			self:HideButtons("partypet", i)
		end                
	end
end

function HealerButtons:OnProfileEnable()
	self:GetBandages()
	self:SetupButtons()
end

function HealerButtons:PARTY_MEMBERS_CHANGED()
	if self.CanChangeButtons then
		self:SetupButtons()
	end
end

function HealerButtons:UNIT_PET()
	if self.CanChangeButtons then
		self:SetupButtons()
	end
end

function HealerButtons:BAG_UPDATE()
	if self.CanChangeButtons then
		self:GetBandages()
		self:SetupButtons()
	end
end

function HealerButtons:PLAYER_REGEN_ENABLED()
	self.CanChangeButtons = false
end

function HealerButtons:PLAYER_REGEN_DISABLED()
	self.CanChangeButtons = true
	self:GetBandages()
	self:SetupButtons()
end

function HealerButtons:SetupButtons()
	self:AttachSpells("player", "Player", "")
	local PartyCount = GetNumPartyMembers()
	for i = 1,4 do
	   if (PartyCount >= i) then 
			self:AttachSpells("party", "PartyMember", i)
			if (self.db.profile.ShowPetButtons and UnitExists("partypet" .. i)) then
				self:AttachSpells("partypet","PartyMember",i)
			else
				self:HideButtons("partypet",i)
			end
		else 
			self:HideButtons("party",i)
			self:HideButtons("partypet",i)
		end                
	end    
end

function HealerButtons:ShowButtons(frame, framenumber)
	local button1 = getglobal("HB_" .. frame .. framenumber .. "Button1")
	local button2 = getglobal("HB_" .. frame .. framenumber .. "Button2")
	local button3 = getglobal("HB_" .. frame .. framenumber .. "Button3")
    button1:Show()
    if (self.db.profile.ButtonCount > 1) then 
        button2:Show()
    else
        button2:Hide()
    end
    if (self.db.profile.ButtonCount > 2) then 
        button3:Show()
    else
        button3:Hide()
    end
end

function HealerButtons:HideButtons(frame, framenumber)
	local button1 = getglobal("HB_" .. frame .. framenumber .. "Button1")
	local button2 = getglobal("HB_" .. frame .. framenumber .. "Button2")
	local button3 = getglobal("HB_" .. frame .. framenumber .. "Button3")
    button1:Hide()
    button2:Hide()
    button3:Hide()
end

function HealerButtons:AttachSpells(frame,frametype,framenumber)
	local button1 = getglobal("HB_" .. frame .. framenumber .. "Button1")
	local button2 = getglobal("HB_" .. frame .. framenumber .. "Button2")
	local button3 = getglobal("HB_" .. frame .. framenumber .. "Button3")
	local framename = frametype .. "Frame" .. framenumber

	if (IsAddOnLoaded("XPerl")) then
		if (string.find(frametype,"Member")) then
			framename = "XPerl_party" .. framenumber
		else
			framename = "XPerl_Player"
		end
	elseif (IsAddOnLoaded("Perl_Player") and string.find(frametype,"Member") == nil) then    
		framename = "Perl_Player_Frame"
	elseif (IsAddOnLoaded("Perl_Party") and  string.find(frametype,"Member")) then 
		framename = "Perl_Party_MemberFrame" .. framenumber
	end

	button1:SetPoint("TOPLEFT", framename, "TOPRIGHT", self.db.profile.ButtonLocs[frame .. framenumber][1], self.db.profile.ButtonLocs[frame .. framenumber][2])
	button2:SetPoint("TOPLEFT", framename, "TOPRIGHT", self.db.profile.ButtonLocs[frame .. framenumber][3], self.db.profile.ButtonLocs[frame .. framenumber][4])
	button3:SetPoint("TOPLEFT", framename, "TOPRIGHT", self.db.profile.ButtonLocs[frame .. framenumber][5], self.db.profile.ButtonLocs[frame .. framenumber][6])
	
	for i = 1,3 do
		self:SetButton(button1, i, i)
		self:SetButton(button2, i+3, i)
		self:SetButton(button3, i+6, i)
	end
	self:ShowButtons(frame, framenumber)
end

function HealerButtons:SetButton(button, index, click)
	button:SetHeight(self.db.profile.ButtonSize[1])
	button:SetWidth(self.db.profile.ButtonSize[2])
	
	if (self.db.profile.SpellTable[self.Class].Type[index] == "BANDAGE") then
		button:SetAttribute("HBType" .. click, "BANDAGE")
		button:SetAttribute("HBName" .. click, self.db.profile.SpellTable[self.Class].Name[index])
		button:SetAttribute("type" .. click, "item")

		local bandageName = self.db.profile.SpellTable[self.Class].Name[index]
		if (string.lower(bandageName) == "bandage") then 
			button:SetAttribute("HBName" .. click, L["Bandage"])
			bandageName = self:GetBestBandage()
		end        
		button:SetAttribute("item"..click, bandageName)
		
		if (click == 1) then
			self:SetItemButtonTexture(button, bandageName)
		end
	else
		button:SetAttribute("HBType" .. click, "SPELL")
		button:SetAttribute("HBName" .. click, self.db.profile.SpellTable[self.Class].Name[index])
		button:SetAttribute("type" .. click, "spell")
		button:SetAttribute("spell" .. click, self.db.profile.SpellTable[self.Class].Name[index])
		
		if (click == 1) then
			self:SetSpellButtonTexture(button, self.db.profile.SpellTable[self.Class].Name[index])
		end
	end
end

function HealerButtons:IsBandage(name)
	return(string.lower(name) == "bandage" or self.bandageInfo[name] ~= nil)
end

function HealerButtons:GetBandages()
    self.bandageInfo[self.bandages.Linen].OnHand = 0
    self.bandageInfo[self.bandages.HeavyLinen].OnHand = 0
    self.bandageInfo[self.bandages.Wool].OnHand = 0
    self.bandageInfo[self.bandages.HeavyWool].OnHand = 0
    self.bandageInfo[self.bandages.Silk].OnHand = 0
    self.bandageInfo[self.bandages.HeavySilk].OnHand = 0
    self.bandageInfo[self.bandages.Mageweave].OnHand = 0
    self.bandageInfo[self.bandages.HeavyMageweave].OnHand = 0
    self.bandageInfo[self.bandages.Runecloth].OnHand = 0
    self.bandageInfo[self.bandages.HeavyRunecloth].OnHand = 0
    self.bandageInfo[self.bandages.Netherweave].OnHand = 0
    self.bandageInfo[self.bandages.HeavyNetherweave].OnHand = 0

    local bag, slot, itemLink, itemName
    local bandageBag = -1
    local bandageSlot = -1
    local healTotal = -1

    for bag = 0,4 do
        for slot = 1,GetContainerNumSlots(bag) do
            itemLink = GetContainerItemLink(bag, slot)
            if itemLink then
                local _, itemCount = GetContainerItemInfo(bag, slot)
                _, _, itemName = string.find(itemLink, "%[(.*)%]")
                if (itemName and self.bandageInfo[itemName]) then
                    self.bandageInfo[itemName].OnHand = itemCount
                    self.bandageInfo[itemName].Bag = bag
                    self.bandageInfo[itemName].Slot = slot
                end
            end
        end
    end
end

function HealerButtons:GetBestBandage()
    local zone = GetZoneText()
    
    if string.find(zone, L["Alterac Valley"]) then 
        if (self.bandageInfo[self.bandages.HeavyRunecloth].OnHand > 0) then 
            return self.bandages.AVHeavyRunecloth
        end
    elseif string.find(zone, L["Arathi Basin"]) then 
        if (self.bandageInfo[self.bandages.HighlandersRunecloth].OnHand > 0) then
            return self.bandages.HighlandersRunecloth
        elseif (self.bandageInfo[self.bandages.HighlandersMageweave].OnHand > 0) then
            return self.bandages.HighlandersMageweave
        elseif (self.bandageInfo[self.bandages.HighlandersSilk].OnHand > 0) then
            return self.bandages.HighlandersSilk
        elseif (self.bandageInfo[self.bandages.DefilersRunecloth].OnHand > 0) then
            return self.bandages.DefilersRunecloth
        elseif (self.bandageInfo[self.bandages.DefilersMageweave].OnHand > 0) then
            return self.bandages.DefilersMageweave
        elseif (self.bandageInfo[self.bandages.DefilersSilk].OnHand > 0) then
            return self.bandages.DefilersSilk
        elseif (self.bandageInfo[self.bandages.ABRunecloth].OnHand > 0) then
            return self.bandages.ABRunecloth
        elseif (self.bandageInfo[self.bandages.ABMageweave].OnHand > 0) then
            return self.bandages.ABMageweave
        elseif (self.bandageInfo[self.bandages.ABSilk].OnHand > 0) then
            return self.bandages.ABSilk
        end
    elseif string.find(zone, L["Warsong Gulch"]) then 
        if (self.bandageInfo[self.bandages.WSGRunecloth].OnHand > 0) then
            return self.bandages.WSGRunecloth
        elseif (self.bandageInfo[self.bandages.WSGMageweave].OnHand > 0) then
            return self.bandages.WSGMageweave
        elseif (self.bandageInfo[self.bandages.WSGSilk].OnHand > 0) then
            return self.bandages.WSGSilk
        end
    end
    if (self.bandageInfo[self.bandages.HeavyNetherweave].OnHand > 0) then
        return self.bandages.HeavyNetherweave
    elseif (self.bandageInfo[self.bandages.Netherweave].OnHand > 0) then
        return self.bandages.Netherweave
    elseif (self.bandageInfo[self.bandages.HeavyRunecloth].OnHand > 0) then
        return self.bandages.HeavyRunecloth
    elseif (self.bandageInfo[self.bandages.Runecloth].OnHand > 0) then
        return self.bandages.Runecloth
    elseif (self.bandageInfo[self.bandages.HeavyMageweave].OnHand > 0) then 
        return self.bandages.HeavyMageweave
    elseif (self.bandageInfo[self.bandages.Mageweave].OnHand > 0) then
        return self.bandages.Mageweave
    elseif (self.bandageInfo[self.bandages.HeavySilk].OnHand > 0) then
        return self.bandages.HeavySilk
    elseif (self.bandageInfo[self.bandages.Silk].OnHand > 0) then
        return self.bandages.Silk
    elseif (self.bandageInfo[self.bandages.HeavyWool].OnHand > 0) then
        return self.bandages.HeavyWool
    elseif (self.bandageInfo[self.bandages.Wool].OnHand > 0) then
        return self.bandages.Wool
    elseif (self.bandageInfo[self.bandages.HeavyLinen].OnHand > 0) then 
        return self.bandages.HeavyLinen
    elseif (self.bandageInfo[self.bandages.Linen].OnHand > 0) then
        return self.bandages.Linen
    else 
        return "NONE"
    end
end

function HealerButtons:UnitClass(unit)    
	local class
	if UnitClass(unit) == L["Priest"] then 
		class = "Priest"
	elseif UnitClass(unit) == L["Druid"] then 
		class = "Druid"
	elseif UnitClass(unit) == L["Paladin"] then 
		class = "Paladin"
	elseif UnitClass(unit) == L["Shaman"] then
		class = "Shaman"
	end
	return class
end

function HealerButtons:IsHealerClass()
	return(self.Class == "Priest" or self.Class == "Paladin" or self.Class == "Druid" or self.Class == "Shaman")
end

function HealerButtons:MakeTooltip(button)
    if (self.db.profile.ShowTooltips) then
		GameTooltip:SetOwner(button, "ANCHOR_NONE")
        if (self.db.profile.TooltipAnchor == "TOPLEFT") then
            GameTooltip:SetPoint("BOTTOMRIGHT", button, "TOPLEFT")
        elseif (self.db.profile.TooltipAnchor == "TOP") then
            GameTooltip:SetPoint("BOTTOM", button, "TOP")
        elseif (self.db.profile.TooltipAnchor == "TOPRIGHT") then
            GameTooltip:SetPoint("BOTTOMLEFT", button, "TOPRIGHT")
        elseif (self.db.profile.TooltipAnchor == "BOTTOMLEFT") then
            GameTooltip:SetPoint("TOPRIGHT", button, "BOTTOMLEFT")
        elseif (self.db.profile.TooltipAnchor == "BOTTOM") then
            GameTooltip:SetPoint("TOP", button, "BOTTOM")
        elseif (self.db.profile.TooltipAnchor == "BOTTOMRIGHT") then
            GameTooltip:SetPoint("TOPLEFT", button, "BOTTOMRIGHT")
        elseif (self.db.profile.TooltipAnchor == "LEFT") then
            GameTooltip:SetPoint("RIGHT", button, "LEFT")
        elseif (self.db.profile.TooltipAnchor == "RIGHT") then
            GameTooltip:SetPoint("LEFT", button, "RIGHT")
        else
			GameTooltip:SetOwner(button, "ANCHOR_CURSOR")
        end

        if (string.find(button:GetName(), "Button1") ) then
        	GameTooltip:AddLine(HIGHLIGHT_FONT_COLOR_CODE .. L["Button"] .. " " .. 1)
        elseif (string.find(button:GetName(), "Button2") ) then
        	GameTooltip:AddLine(HIGHLIGHT_FONT_COLOR_CODE .. L["Button"] .. " " .. 2)
        else
        	GameTooltip:AddLine(HIGHLIGHT_FONT_COLOR_CODE .. L["Button"] .. " " .. 3)
		end

        if (string.len(button:GetAttribute("HBName1")) > 0 and not (string.lower(button:GetAttribute("HBName1")) == "none") ) then 
            GameTooltip:AddLine(NORMAL_FONT_COLOR_CODE .. L["Left: "] .. LIGHTYELLOW_FONT_COLOR_CODE .. button:GetAttribute("HBName1"))
        end

        if (string.len(button:GetAttribute("HBName2")) > 0 and not (string.lower(button:GetAttribute("HBName2")) == "none") ) then 
            GameTooltip:AddLine(NORMAL_FONT_COLOR_CODE .. L["Right: "] .. LIGHTYELLOW_FONT_COLOR_CODE .. button:GetAttribute("HBName2"))
        end

        if (string.len(button:GetAttribute("HBName3")) > 0 and not (string.lower(button:GetAttribute("HBName3")) == "none") ) then 
            GameTooltip:AddLine(NORMAL_FONT_COLOR_CODE .. L["Middle: "] .. LIGHTYELLOW_FONT_COLOR_CODE .. button:GetAttribute("HBName3"))
        end

        GameTooltip:Show()
    end
end

function HealerButtons:ResetConfigSavedLocation()
   local fromWhich = HealerButtons.PreferenceDefaults.ButtonLocs
   for i = 1,6 do
       self.db.profile.ButtonLocs.player[i] = fromWhich.player[i]
       for j = 1,4 do
           local party = "party" .. j
           local partypet = "partypet" .. j
           self.db.profile.ButtonLocs[party][i] = fromWhich[party][i]
           self.db.profile.ButtonLocs[partypet][i] = fromWhich[partypet][i]
       end
   end
   self:SetupButtons()
   self:Print(L["Buttons have been reset to the default positions."])
end

function HealerButtons:Points(button, name, offset)
	if (string.find(name, "party1")) then
		name = "party1"
	elseif (string.find(name, "party2")) then
		name = "party2"
	elseif (string.find(name, "party3")) then
		name = "party3"
	elseif (string.find(name, "party4")) then
		name = "party4"
	elseif (string.find(name, "partypet1")) then
		name = "partypet1"
	elseif (string.find(name, "partypet2")) then
		name = "partype21"
	elseif (string.find(name, "partypet3")) then
		name = "partypet3"
	elseif (string.find(name, "partypet4")) then
		name = "partypet4"
	else
		name = "player"
	end
	
	local x = self.db.profile.ButtonLocs[name][offset + 1] - (self.ButtonNow[1] - button:GetLeft())
	local y = self.db.profile.ButtonLocs[name][offset + 2] - (self.ButtonNow[2] - button:GetTop())

	return name, x, y
end

function HealerButtons:OnDragStart(button)
	if (self.db.profile.Movable and not self.BeingDragged) then
        self.ButtonNow = { button:GetLeft(), button:GetTop() }
		button:StartMoving()
		self.BeingDragged = true
	end
end

function HealerButtons:OnDragStop(button)
    local name = button:GetName()
    local x
    local y
    local height = button:GetHeight()
    local offset = 0

	if (self.BeingDragged) then
		button:StopMovingOrSizing()
		self.BeingDragged = false
        if (string.find(name, "Button1")) then
            offset = 0
        elseif (string.find(name, "Button2")) then
            offset = 2
        else 
            offset = 4
        end

        name, x, y = self:Points(button, name, offset)
        self.db.profile.ButtonLocs[name][offset + 1] = x
        self.db.profile.ButtonLocs[name][offset + 2] = y
	end
end

function HealerButtons:SetSpellButtonTexture(button, spellNameMatch)
	local i = 1
	while (true) do
		local spellName, spellRank = GetSpellName(i, BOOKTYPE_SPELL)
		if (not spellName) then
			button:SetNormalTexture("Interface\\Icons\\INV_Misc_QuestionMark")
			do break end
		elseif (spellName == spellNameMatch) then
			local texture = GetSpellTexture(i, BOOKTYPE_SPELL)
			button:SetNormalTexture(texture)
			do break end
		end
		i = i + 1
	end
end

function HealerButtons:SetItemButtonTexture(button, itemNameMatch)
    local _, bag, slot, itemLink, itemName, texture
    for bag=0,4 do
        for slot=1, GetContainerNumSlots(bag) do
            itemLink = GetContainerItemLink(bag, slot)
            if (itemLink) then
                _, _, itemName = string.find(itemLink, "%[(.*)%]")
				if (itemName == itemNameMatch) then
					texture, _, _, _, _ = GetContainerItemInfo(bag, slot)
					button:SetNormalTexture(texture)
					return
                end
            end
        end
    end
end

function HealerButtons:GetButtonIndex(code, button, click)
	local index = -1
	code = string.lower(code)
	if code == "1r" or (button == 1 and click == "RightButton") then    
		index = 2
	elseif code == "1m" or (button == 1 and click == "MiddleButton") then
		index = 3
	elseif code == "1l" or (button == 1 and click == "LeftButton") then
		index = 1
	elseif code == "2r" or (button == 2 and click == "RightButton") then
		index = 5
	elseif code == "2m" or (button == 2 and click == "MiddleButton") then
		index = 6
	elseif code == "2l" or (button == 2 and click == "LeftButton") then
		index = 4
	elseif code == "3r" or (button == 3 and click == "RightButton") then
		index = 8
	elseif code == "3m" or (button == 3 and click == "MiddleButton") then
		index = 9
	elseif code == "3l" or (button == 3 and click == "LeftButton") then
		index = 7
	end
	return index
end

function HealerButtons:SetBinding(code, n1, n2, n3)    
    local index = self:GetButtonIndex(code)
    if (index > 0) then
		local spellname = n1
		if n2 then
			spellname = spellname .. " " .. n2
		end
		if n3 then
			spellname = spellname .. " " .. n3
		end

        self.db.profile.SpellTable[self.Class].Name[index] = spellname
        if self:IsBandage(spellname) then
            self.db.profile.SpellTable[self.Class].Type[index] = "BANDAGE"
        else
            self.db.profile.SpellTable[self.Class].Type[index] = "SPELL"
        end
        self:SetupButtons()
        self:Print(code .. L[" button has been set to: "] .. spellname)
    end
end

function HealerButtons:VerifyBinding(code)    
	return(self:GetButtonIndex(code) > 0)
end

function HealerButtons:GetButtonCount()
	return self.db.profile.ButtonCount
end

function HealerButtons:SetButtonCount(v)
	self.db.profile.ButtonCount = v
	self:SetupButtons()
end

function HealerButtons:GetButtonSize()
	return self.db.profile.ButtonSize[1]
end

function HealerButtons:SetButtonSize(v)
	self.db.profile.ButtonSize = { math.min(v), math.min(v) }
	self:SetupButtons()
end

function HealerButtons:GetMovable()
	return self.db.profile.Movable
end

function HealerButtons:SetMovable(v)
	self.db.profile.Movable = v
end

function HealerButtons:GetShowPetButtons()
	return self.db.profile.ShowPetButtons
end

function HealerButtons:SetShowPetButtons(v)
	self.db.profile.ShowPetButtons = v
end

function HealerButtons:GetShowTooltips()
	return self.db.profile.ShowTooltips
end

function HealerButtons:SetShowTooltips(v)
	self.db.profile.ShowTooltips = v
end

function HealerButtons:GetTooltipAnchor()
	return self.db.profile.TooltipAnchor
end

function HealerButtons:SetTooltipAnchor(v)
	self.db.profile.TooltipAnchor = v
end

